
import MetalKit

class Node {
    var name: String = "untitled"
    var position: float3 = [0, 0, 0]
    var rotation: float3 = [0, 0, 0]
    var scale: float3 = [1, 1, 1]
    
    var modelMatrix: float4x4 {
        let translateMatrix = float4x4(translation: position)
        let rotateMatrix = float4x4(rotation: rotation)
        let scaleMatrix = float4x4(scaling: scale)
        return translateMatrix * rotateMatrix * scaleMatrix
    }
    
    var parent: Node?
    var children: [Node] = []
    
    final func add(childNode: Node) {
        children.append(childNode)
        childNode.parent = self
    }
    
    final func remove(childNode: Node) {
        // removes parent-child relationship between childNode and its own children, the parent node (or grandparent node) becomes a parent node to child node's children
        for child in childNode.children {
            child.parent = self
            children.append(child)
        }
        
        // empties children array of child
        childNode.children = []
        // finds index of children where item is childNode and removes it
        guard let index = (children.firstIndex { $0 === childNode}) else { return }
        children.remove(at: index)
        childNode.parent = nil
    }
}
